@@ -5,90 +5,120 @@
 #' business days calculations.
 #'
 #' @param holidays a vector of Dates which contains the holidays
+#' @param start.date the date which calendar starts
+#' @param end.date the date which calendar ends
 #' @export
 #' @examples
 #' # holidays has iso-formated dates
 #' data(holidaysANBIMA)
 #' cal <- Calendar(holidaysANBIMA)
-Calendar <- function (holidays) {
+#' # For empty calendar just pass nothing
+#' cal <- Calendar()
+Calendar <- function (holidays=integer(0), start.date='1970-01-01', end.date='2071-01-01') {
 	that <- list()
-	dates <- seq(from=min(holidays), to=max(holidays), by='day')
+	start.date <- as.Date(start.date)
+	end.date <- as.Date(end.date)
+	if (length(holidays) != 0) {
+		start.date <- as.Date(min(holidays))
+		end.date <- as.Date(max(holidays))
+	}
+	dates <- seq(from=start.date, to=end.date, by='day')
 	n.dates <- as.integer(dates)
 	n.holidays <- as.integer(holidays)
 	.is.bizday <- vapply(n.dates, function(.) {
-	    wday <- .%%7
-	    ! ( wday == 2 || wday == 3 || any(. == n.holidays))
+		wday <- .%%7
+		! ( wday == 2 || wday == 3 || any(. == n.holidays))
 	}, logical(1))
 	n.bizdays <- n.dates[.is.bizday]
 	idx <- as.integer(1)
 	index <- vapply(.is.bizday, function(.) {
-	    prev.idx <- idx
-	    idx <<- prev.idx + as.integer(.)
-	    prev.idx
+		prev.idx <- idx
+		idx <<- prev.idx + as.integer(.)
+		prev.idx
 	}, integer(1))
 	# class attributes
 	bizdays <- dates[.is.bizday]
 	.adjust.next <- function(date) {
-	    date <- as.Date(date)
-	    while ( ! .is.bizday[dates == date] ) date <- date + 1
-	    date
+		date <- as.Date(date)
+		while ( ! .is.bizday[dates == date] ) date <- date + 1
+		date
 	}
 	.adjust.previous <- function(date) {
-	    date <- as.Date(date)
-	    while ( ! .is.bizday[dates == date] ) date <- date - 1
-	    date
+		date <- as.Date(date)
+		while ( ! .is.bizday[dates == date] ) date <- date - 1
+		date
+	}
+	.adjust <- function(dates, .adjust.FUN) {
+		if ( ! any(as.Date(dates) >= start.date & as.Date(dates) <= end.date) )
+			stop('Given date out of range.')
+		o.dates <- integer(length(dates))
+		for (i in seq_along(dates)) {
+			o.dates[i] <- .adjust.FUN(dates[i])
+		}
+		as.Date(o.dates, origin='1970-01-01')
+	}
+	.bizdays <- function(from, to) {
+		from.idx <- index[dates %in% that$adjust.next(from)]
+		to.idx <- index[dates %in% that$adjust.previous(to)]
+		stopifnot(length(from.idx) == length(to.idx))
+		to.idx - from.idx
+	}
+	that$adjust.next <- function(dates) {
+		.adjust(dates, .adjust.next)
+	}
+	that$adjust.previous <- function(dates) {
+		.adjust(dates, .adjust.previous)
 	}
-    .adjust <- function(dates, .adjust.FUN) {
-        o.dates <- integer(length(dates))
-        for (i in seq_along(dates)) {
-            o.dates[i] <- .adjust.FUN(dates[i])
-        }
-        as.Date(o.dates, origin='1970-01-01')
-    }
-    .bizdays <- function(from, to) {
-        from.idx <- index[dates %in% that$adjust.next(from)]
-        to.idx <- index[dates %in% that$adjust.previous(to)]
-        stopifnot(length(from.idx) == length(to.idx))
-        to.idx - from.idx
-    }
-    that$adjust.next <- function(dates) {
-        .adjust(dates, .adjust.next)
-    }
-    that$adjust.previous <- function(dates) {
-        .adjust(dates, .adjust.previous)
-    }
 	that$bizdays <- function(from, to) {
-        stopifnot(all(from <= to))
-        stopifnot(length(from) == length(to))
-        bd <- integer(length(from))
-        for (i in seq_along(from)) {
-            bd[i] <- .bizdays(from[i], to[i])
-        }
-        bd
+		if ( ! any(as.Date(from) >= start.date & as.Date(from) <= end.date) )
+			stop('Given date out of range.')
+		if ( ! any(as.Date(to) >= start.date & as.Date(to) <= end.date) )
+			stop('Given date out of range.')
+		if ( ! all(from <= to) )
+			stop('All from dates must be greater than all to dates.')
+		if (length(from) != length(to))
+			stop('from and to must have the same length.')
+		bd <- integer(length(from))
+		for (i in seq_along(from)) {
+			bd[i] <- .bizdays(from[i], to[i])
+		}
+		bd
+	}
+	that$is.bizday <- function(date) {
+		if ( ! any(as.Date(date) >= start.date & as.Date(date) <= end.date) )
+			stop('Given date out of range.')
+		.is.bizday[dates %in% date]
 	}
-	that$is.bizday <- function(date) .is.bizday[dates %in% date]
 	that$seq <- function(from, to) {
+		if ( ! any(as.Date(from) >= start.date & as.Date(from) <= end.date) )
+			stop('Given date out of range.')
+		if ( ! any(as.Date(to) >= start.date & as.Date(to) <= end.date) )
+			stop('Given date out of range.')
+		if ( ! all(from <= to) )
+			stop('All from dates must be greater than all to dates.')
 		bizdays[which(bizdays >= from & bizdays <= to)]
 	}
     that$offset <- function(date, n) {
-        if (n >= 0) {
-            adjust <- function(date) .adjust(date, .adjust.next)
-            date <- adjust(date)
-            inc <- 1
-        } else {
-            adjust <- function(date) .adjust(date, .adjust.previous)
-            date <- adjust(date)
-            inc <- -1
-            n <- abs(n)
-        }
-        i <- 0
-        while (i < n) {
-            date <- date + inc
-            date <- adjust(date)
-            i <- i + 1
-        }
-        date
-    }
+		if ( ! any(as.Date(date) >= start.date & as.Date(date) <= end.date) )
+			stop('Given date out of range.')
+		if (n >= 0) {
+			adjust <- function(date) .adjust(date, .adjust.next)
+			date <- adjust(date)
+			inc <- 1
+		} else {
+			adjust <- function(date) .adjust(date, .adjust.previous)
+			date <- adjust(date)
+			inc <- -1
+			n <- abs(n)
+		}
+		i <- 0
+		while (i < n) {
+			date <- date + inc
+			date <- adjust(date)
+			i <- i + 1
+		}
+		date
+	}
 	class(that) <- 'Calendar'
 	return(that)
 }
